<?php

/**
 * This file is part of Zwii.
 *
 * For full copyright and license information, please see the LICENSE
 * file that was distributed with this source code.
 *
 * @author Rémi Jean <remi.jean@outlook.com>
 * @copyright Copyright (C) 2008-2018, Rémi Jean
 * @license GNU General Public License, version 3
 * @link http://zwiicms.com/
 */

class livredor extends common {

	public static $actions = [
		'add' => self::GROUP_VISITOR,
		'init' => self::GROUP_MODERATOR,
		'config' => self::GROUP_MODERATOR,
		'delete' => self::GROUP_MODERATOR,
		'edit' => self::GROUP_VISITOR,
		'index' => self::GROUP_VISITOR
	];

	public static $articles = [];
	
	public static $livres = [];

	public static $comments = [];

	public static $pages;

	public static $states = [
		false => 'Brouillon',
		true => 'Publié'
	];
	
	public static $ItemsList = [
		4 => '4 Messages',
		8 => '8 Messages',
		12 => '12 Messages',
		16 => '16 Messages',
		22 => '22  Messages'
	];

	public static $users = [];

	
	const VERSION = '1.2.5';
	const REALNAME = 'Livre d\'or';
	const DELETE = true;
	const UPDATE = '0.0';
	const DATADIRECTORY = '';
	
	
	/**
	* Initialisation du module livre d'or dans module.json
	*/
	private function init() {
		// Si Données "itemsperPage" du module sont absentes
		if ($this->getData(['module', $this->getUrl(0), 'config' ]) === null) {
			//on ajoute par défaut 4 ligne par page
			$this->setData(['module', $this->getUrl(0), 'config',[
				'itemsperPage' => '4'
				]]);
			}
	}
	
	/**
	 * Édition
	 */
	public function add() {
		// Soumission du formulaire
		if($this->isPost()) {
			// Vérification avec captcha
			if(
				$this->getUser('password') !== $this->getInput('ZWII_USER_PASSWORD')
				AND password_verify($this->getInput('livreAddCaptcha', helper::FILTER_INT), $this->getInput('livreAddCaptchaResult') ) === false )
					{
						self::$inputNotices['livreAddCaptcha'] = 'Valeur incorrecte';
					}else{

			// Incrémente l'id du message
			$livreId = helper::increment($this->getInput('livreAddNom', helper::FILTER_ID), $this->getData(['page']));
			$livreId = helper::increment($livreId, (array) $this->getData(['module', $this->getUrl(0)]));
			$livreId = helper::increment($livreId, array_keys(self::$actions));
			// Création du message
			$this->setData(['module', $this->getUrl(0),'post', $livreId, [
				'content' => $this->getInput('livreAddContent', null),
				'publishedOn' => $this->getInput('livreAddPublishedOn', helper::FILTER_DATETIME, false),
				'state' => $this->getInput('livreAddState', helper::FILTER_BOOLEAN),
				'Nom' => $this->getInput('livreAddNom', helper::FILTER_STRING_SHORT, true),
				'email' => $this->getInput('livreAddEmail', helper::FILTER_STRING_SHORT, true),
				'userId' => $this->getInput('livreAddUserId', helper::FILTER_ID, false)
				
			]]);
					// Envoi d'une notification
					
					// déclaration des variables
					$nom= $this->getInput('livreAddNom'); // variable nom
					$mail= $this->getInput('livreAddEmail'); // variable Email
					$contenu= $this->getInput('livreAddContent'); // variable du contenu
					$group = $this->getData(['module', $this->getUrl(0), 'config', 'group']);// variable du groupe admin
						
					// déclaration du groupe admin
					$to = [];
						if ($group = 3){
							foreach($this->getData(['user']) as $userId => $user) {
								if($user['group'] === $group) {
									$to[] = $user['mail'];
								}
							}
						}
					// si $to n'est pas vide
						if($to) {
									
					// Envoi le mail
							$sent = $this->sendMail(
								$to,
								'Nouveau Message du livre d or',
								'Nouveau message posté par ' . ' <strong>' . $nom . '</strong> sur le livre d or,<br><br>' .
								'Email de : '  . $mail . '<br><br>' .
								'Message : <br>' . $contenu . '<br><br>'
								);
						}
						// Valeurs en sortie
			$this->addOutput([
				'redirect' => helper::baseUrl() . $this->getUrl(0) . '/',
				'notification' => 'Message créé',
				'state' => true
			]);
		}				
		}
		// Liste des utilisateurs
		self::$users = helper::arrayCollumn($this->getData(['user']), 'firstname');
		ksort(self::$users);
		foreach(self::$users as $userId => &$userFirstname) {
			$userFirstname = $userFirstname . ' ' . $this->getData(['user', $userId, 'lastname']);
		}
		unset($userFirstname);
		// Valeurs en sortie
		$this->addOutput([
			'Nom' => 'Nouveau message',
			'vendor' => [
				'flatpickr',
				'tinymce'
			],
			'view' => 'add'
		]);
	}
	



	/**
	 * Configuration
	 */
	public function config() {
		// Initialisation d'un nouveau module
		$this->init();
		
		if($this->isPost()) {
			$this->setData(['module', $this->getUrl(0), 'config',[
				'itemsperPage' => $this->getInput('livreConfigItemsperPage', helper::FILTER_INT,true)
				]]);
			// Valeurs en sortie
			$this->addOutput([
				'redirect' => helper::baseUrl() . $this->getUrl(0) . '/config',
				'notification' => 'Modifications enregistrées',
				'state' => true
			]);
		} 
		// Ids des messages par ordre de publication
		$livreIds = array_keys(helper::arrayCollumn($this->getData(['module', $this->getUrl(0),'post']), 'publishedOn', 'SORT_DESC'));
		// Pagination
		$pagination = helper::pagination($livreIds, $this->getUrl(),$this->getData(['module', $this->getUrl(0), 'config', 'itemsperPage']) );
			
		// Liste des pages
		self::$pages = $pagination['pages']; 
		// Messages en fonction de la pagination
		for($i = $pagination['first']; $i < $pagination['last']; $i++) {
			// Met en forme le tableau
			self::$livres[] = [
				$this->getData(['module', $this->getUrl(0),'post', $livreIds[$i], 'Nom']),
				utf8_encode(strftime('%d %B %Y', $this->getData(['module', $this->getUrl(0),'post', $livreIds[$i], 'publishedOn']))),
				self::$states[$this->getData(['module', $this->getUrl(0),'post', $livreIds[$i], 'state'])],
				template::button('livreConfigEdit' . $livreIds[$i], [
					'href' => helper::baseUrl() . $this->getUrl(0) . '/edit/' . $livreIds[$i] . '/' . $_SESSION['csrf'],
					'value' => template::ico('pencil')
				]),
				template::button('livreConfigDelete' . $livreIds[$i], [
					'class' => 'livreConfigDelete buttonRed',
					'href' => helper::baseUrl() . $this->getUrl(0) . '/delete/' . $livreIds[$i] . '/' . $_SESSION['csrf'],
					'value' => template::ico('cancel')
				])
			];
		}
		// Valeurs en sortie
		$this->addOutput([
			'Nom' => 'Configuration du module',
			'view' => 'config'
		]);
	
	}
	/**
	 * Suppression
	 */
	public function delete() {
		if($this->getData(['module', $this->getUrl(0),'post', $this->getUrl(2)]) === null) {
			// Valeurs en sortie
			$this->addOutput([
				'access' => false
			]);
		}
		// Jeton incorrect
		elseif ($this->getUrl(3) !== $_SESSION['csrf']) {
			// Valeurs en sortie
			$this->addOutput([
				'redirect' => helper::baseUrl()  . $this->getUrl(0) . '/config',
				'notification' => 'Action non autorisée'
			]);
		}			
		// Suppression
		else {
			$this->deleteData(['module', $this->getUrl(0),'post', $this->getUrl(2)]);
			// Valeurs en sortie
			$this->addOutput([
				'redirect' => helper::baseUrl() . $this->getUrl(0) . '/config',
				'notification' => 'Message supprimé',
				'state' => true
			]);
		}
	}

	/**
	 * Édition
	 */
	public function edit() {
		// Jeton incorrect
		if ($this->getUrl(3) !== $_SESSION['csrf']) {
			// Valeurs en sortie
			$this->addOutput([
				'redirect' => helper::baseUrl() . $this->getUrl(0) . '/config',
				'notification' => 'Action  non autorisée'
			]);
		}			
		// Le message n'existe pas
		if($this->getData(['module', $this->getUrl(0),'post', $this->getUrl(2)]) === null) {
			// Valeurs en sortie
			$this->addOutput([
				'access' => false
			]);
		}
		// Le message existe
		else {
			// Soumission du formulaire
			if($this->isPost()) {	
				$livreId = $this->getInput('livreEditNom', helper::FILTER_ID, true);
				// Incrémente le nouvel id du message
				if($livreId !== $this->getUrl(2)) {
					$livreId = helper::increment($livreId, $this->getData(['page']));
					$livreId = helper::increment($livreId, $this->getData(['module', $this->getUrl(0),'post']));
					$livreId = helper::increment($livreId, array_keys(self::$actions));
				}
				$this->setData(['module', $this->getUrl(0),'post', $livreId, [
					'content' => $this->getInput('livreEditContent', null),
					'publishedOn' => $this->getInput('livreEditPublishedOn', helper::FILTER_DATETIME, true),
					'state' => $this->getInput('livreEditState', helper::FILTER_BOOLEAN),
					'Nom' => $this->getInput('livreEditNom', helper::FILTER_STRING_SHORT, true),
					'email' => $this->getInput('livreEditEmail', helper::FILTER_STRING_SHORT, true),
					'userId' => $this->getInput('livreEditUserId', helper::FILTER_ID, true)
				]]);
				// Supprime l'ancien message
				if($livreId !== $this->getUrl(2)) {
					$this->deleteData(['module', $this->getUrl(0),'post', $this->getUrl(2)]);
				}
				
				// Valeurs en sortie
				$this->addOutput([
					'redirect' => helper::baseUrl() . $this->getUrl(0) . '/config',
					'notification' => 'Modifications enregistrées',
					'state' => true
				]);
			}
			// Liste des utilisateurs
			self::$users = helper::arrayCollumn($this->getData(['user']), 'firstname');
			ksort(self::$users);
			foreach(self::$users as $userId => &$userFirstname) {
				$userFirstname = $userFirstname . ' ' . $this->getData(['user', $userId, 'lastname']);
			}
			unset($userFirstname);
			// Valeurs en sortie
			$this->addOutput([
				'Nom' => $this->getData(['module', $this->getUrl(0),'post', $this->getUrl(2), 'Nom']),
				'vendor' => [
					'flatpickr',
					'tinymce'
				],
				'view' => 'edit'
			]);
		}
	}

	/**
	 * Accueil (deux affichages en un pour éviter une url à rallonge)
	 */
	public function index() {
	// Initialisation d'un nouveau module
		$this->init();
	// Affichage d'un message
		if(
			$this->getUrl(1)
			// Protection pour la pagination, un ID ne peut pas être un entier, une page oui
			AND intval($this->getUrl(1)) === 0
		) {
			// Le message n'existe pas
			if($this->getData(['module', $this->getUrl(0),'post', $this->getUrl(1)]) === null) {
				// Valeurs en sortie
				$this->addOutput([
					'access' => false
				]);
			}
			// Si Le message existe
			else {
			
				//on affiche la valeur du message concerné
				$this->addOutput([
					'showBarEditButton' => true,
					'Nom' => $this->getData(['module', $this->getUrl(0),'post', $this->getUrl(1), 'Nom']),
					'view' => 'article'
				]);
			}
    
		}
		// Liste des messages
		else {
			// Ids des messages par ordre de publication
			$livreIdsPublishedOns = helper::arrayCollumn($this->getData(['module', $this->getUrl(0),'post']), 'publishedOn', 'SORT_DESC');
			$livreIdsStates = helper::arrayCollumn($this->getData(['module', $this->getUrl(0),'post']), 'state', 'SORT_DESC');
			$livreIds = [];
			foreach($livreIdsPublishedOns as $livreId => $livrePublishedOn) {
				if($livrePublishedOn <= time() AND $livreIdsStates[$livreId]) {
					$livreIds[] = $livreId;
				}
			}
			// Pagination
			//$pagination = helper::pagination($livreIds, $this->getUrl(),$this->getData(['config','itemsperPage']));
			$pagination = helper::pagination($livreIds, $this->getUrl(),$this->getData(['module', $this->getUrl(0),'config', 'itemsperPage']));
			
			// Liste des pages
			self::$pages = $pagination['pages'];
			// messages en fonction de la pagination
			for($i = $pagination['first']; $i < $pagination['last']; $i++) {
				self::$livres[$livreIds[$i]] = $this->getData(['module', $this->getUrl(0),'post', $livreIds[$i]]);
			}
			// Affiche la liste des messages
			$this->addOutput([
				'showBarEditButton' => true,
				'showPageContent' => true,
				'view' => 'index'
			]);
		}

	}
}
